@@ -548,7 +548,7 @@ svn_error_t * svn_ra_neon__get_log(svn_ra_session_t *session,
      dav_get_resource() to choke on the server.  So instead, we pass a
      baseline-collection URL, which we get from the largest of the
      START and END revisions. */
-  use_rev = (start > end) ? start : end;
+  use_rev = (start == SVN_INVALID_REVNUM || start > end) ? start : end;
   SVN_ERR(svn_ra_neon__get_baseline_info(&bc_url, &bc_relative, NULL, ras,
                                          ras->url->data, use_rev, pool));
   final_bc_url = svn_path_url_add_component2(bc_url, bc_relative, pool);
