@@ -2,17 +2,22 @@
  * config.c :  reading configuration information
  *
  * ====================================================================
- * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
+ *    Licensed to the Apache Software Foundation (ASF) under one
+ *    or more contributor license agreements.  See the NOTICE file
+ *    distributed with this work for additional information
+ *    regarding copyright ownership.  The ASF licenses this file
+ *    to you under the Apache License, Version 2.0 (the
+ *    "License"); you may not use this file except in compliance
+ *    with the License.  You may obtain a copy of the License at
  *
- * This software is licensed as described in the file COPYING, which
- * you should have received as part of this distribution.  The terms
- * are also available at http://subversion.tigris.org/license-1.html.
- * If newer versions of this license are posted there, you may use a
- * newer version instead, at your option.
+ *      http://www.apache.org/licenses/LICENSE-2.0
  *
- * This software consists of voluntary contributions made by many
- * individuals.  For exact contribution history, see the revision
- * history and logs, available at http://subversion.tigris.org/.
+ *    Unless required by applicable law or agreed to in writing,
+ *    software distributed under the License is distributed on an
+ *    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ *    KIND, either express or implied.  See the License for the
+ *    specific language governing permissions and limitations
+ *    under the License.
  * ====================================================================
  */
 
@@ -73,18 +78,34 @@ struct cfg_option_t
 
 
 svn_error_t *
-svn_config_read(svn_config_t **cfgp, const char *file,
-                svn_boolean_t must_exist, apr_pool_t *pool)
+svn_config_create(svn_config_t **cfgp,
+                  svn_boolean_t section_names_case_sensitive,
+                  apr_pool_t *result_pool)
 {
-  svn_config_t *cfg = apr_palloc(pool, sizeof(*cfg));
+  svn_config_t *cfg = apr_palloc(result_pool, sizeof(*cfg));
+
+  cfg->sections = apr_hash_make(result_pool);
+  cfg->pool = result_pool;
+  cfg->x_pool = svn_pool_create(result_pool);
+  cfg->x_values = FALSE;
+  cfg->tmp_key = svn_stringbuf_create("", result_pool);
+  cfg->tmp_value = svn_stringbuf_create("", result_pool);
+  cfg->section_names_case_sensitive = section_names_case_sensitive;
+
+  *cfgp = cfg;
+  return SVN_NO_ERROR;
+}
+
+svn_error_t *
+svn_config_read2(svn_config_t **cfgp, const char *file,
+                 svn_boolean_t must_exist,
+                 svn_boolean_t section_names_case_sensitive,
+                 apr_pool_t *pool)
+{
+  svn_config_t *cfg;
   svn_error_t *err;
 
-  cfg->sections = apr_hash_make(pool);
-  cfg->pool = pool;
-  cfg->x_pool = svn_pool_create(pool);
-  cfg->x_values = FALSE;
-  cfg->tmp_key = svn_stringbuf_create("", pool);
-  cfg->tmp_value = svn_stringbuf_create("", pool);
+  SVN_ERR(svn_config_create(&cfg, section_names_case_sensitive, pool));
 
   /* Yes, this is platform-specific code in Subversion, but there's no
      practical way to migrate it into APR, as it's simultaneously
@@ -138,7 +159,7 @@ read_all(svn_config_t **cfgp,
 #ifdef WIN32
   if (sys_registry_path)
     {
-      SVN_ERR(svn_config_read(cfgp, sys_registry_path, FALSE, pool));
+      SVN_ERR(svn_config_read2(cfgp, sys_registry_path, FALSE, FALSE, pool));
       red_config = TRUE;
     }
 #endif /* WIN32 */
@@ -149,7 +170,7 @@ read_all(svn_config_t **cfgp,
         SVN_ERR(svn_config_merge(*cfgp, sys_file_path, FALSE));
       else
         {
-          SVN_ERR(svn_config_read(cfgp, sys_file_path, FALSE, pool));
+          SVN_ERR(svn_config_read2(cfgp, sys_file_path, FALSE, FALSE, pool));
           red_config = TRUE;
         }
     }
@@ -163,7 +184,8 @@ read_all(svn_config_t **cfgp,
         SVN_ERR(svn_config_merge(*cfgp, usr_registry_path, FALSE));
       else
         {
-          SVN_ERR(svn_config_read(cfgp, usr_registry_path, FALSE, pool));
+          SVN_ERR(svn_config_read2(cfgp, usr_registry_path,
+                                   FALSE, FALSE, pool));
           red_config = TRUE;
         }
     }
@@ -175,7 +197,7 @@ read_all(svn_config_t **cfgp,
         SVN_ERR(svn_config_merge(*cfgp, usr_file_path, FALSE));
       else
         {
-          SVN_ERR(svn_config_read(cfgp, usr_file_path, FALSE, pool));
+          SVN_ERR(svn_config_read2(cfgp, usr_file_path, FALSE, FALSE, pool));
           red_config = TRUE;
         }
     }
@@ -311,7 +333,7 @@ svn_config_merge(svn_config_t *cfg, const char *file,
      ### We could use a tmp subpool for this, since merge_cfg is going
      to be tossed afterwards.  Premature optimization, though? */
   svn_config_t *merge_cfg;
-  SVN_ERR(svn_config_read(&merge_cfg, file, must_exist, cfg->pool));
+  SVN_ERR(svn_config_read2(&merge_cfg, file, must_exist, FALSE, cfg->pool));
 
   /* Now copy the new options into the original table. */
   for_each_option(merge_cfg, cfg, merge_cfg->pool, merge_callback);
@@ -355,7 +377,7 @@ make_hash_key(char *key)
 {
   register char *p;
   for (p = key; *p != 0; ++p)
-    *p = apr_tolower(*p);
+    *p = (char)apr_tolower(*p);
   return key;
 }
 
@@ -371,7 +393,8 @@ find_option(svn_config_t *cfg, const char *section, const char *option,
 
   /* Canonicalize the hash key */
   svn_stringbuf_set(cfg->tmp_key, section);
-  make_hash_key(cfg->tmp_key->data);
+  if (! cfg->section_names_case_sensitive)
+    make_hash_key(cfg->tmp_key->data);
 
   sec_ptr = apr_hash_get(cfg->sections, cfg->tmp_key->data,
                          cfg->tmp_key->len);
@@ -602,7 +625,10 @@ svn_config_set(svn_config_t *cfg,
       /* Even the section doesn't exist. Create it. */
       sec = apr_palloc(cfg->pool, sizeof(*sec));
       sec->name = apr_pstrdup(cfg->pool, section);
-      sec->hash_key = make_hash_key(apr_pstrdup(cfg->pool, section));
+      if(cfg->section_names_case_sensitive)
+        sec->hash_key = sec->name;
+      else
+        sec->hash_key = make_hash_key(apr_pstrdup(cfg->pool, section));
       sec->options = apr_hash_make(cfg->pool);
       apr_hash_set(cfg->sections, sec->hash_key, APR_HASH_KEY_STRING, sec);
     }
@@ -625,41 +651,25 @@ static svn_error_t *
 get_bool(svn_boolean_t *boolp, const char *input, svn_boolean_t default_value,
          const char *section, const char *option)
 {
-  if (input == NULL)
-    {
-      *boolp = default_value;
-    }
-  else if (0 == svn_cstring_casecmp(input, SVN_CONFIG_TRUE)
-           || 0 == svn_cstring_casecmp(input, "yes")
-           || 0 == svn_cstring_casecmp(input, "on")
-           || 0 == strcmp(input, "1"))
-    {
-      *boolp = TRUE;
-    }
-  else if (0 == svn_cstring_casecmp(input, SVN_CONFIG_FALSE)
-           || 0 == svn_cstring_casecmp(input, "no")
-           || 0 == svn_cstring_casecmp(input, "off")
-           || 0 == strcmp(input, "0"))
-    {
-      *boolp = FALSE;
-    }
-  else  /* unrecognized value */
-    {
-      if (section)
-        {
-          return svn_error_createf(SVN_ERR_BAD_CONFIG_VALUE, NULL,
-                                   _("Config error: invalid boolean "
-                                     "value '%s' for '[%s] %s'"),
-                                   input, section, option);
-        }
-      else
-        {
-          return svn_error_createf(SVN_ERR_BAD_CONFIG_VALUE, NULL,
-                                   _("Config error: invalid boolean "
-                                     "value '%s' for '%s'"),
-                                   input, option);
-        }
-    }
+  svn_tristate_t value = svn_tristate__from_word(input);
+
+  if (value == svn_tristate_true)
+    *boolp = TRUE;
+  else if (value == svn_tristate_false)
+    *boolp = FALSE;
+  else if (input == NULL) /* no value provided */
+    *boolp = default_value;
+
+  else if (section) /* unrecognized value */
+    return svn_error_createf(SVN_ERR_BAD_CONFIG_VALUE, NULL,
+                             _("Config error: invalid boolean "
+                               "value '%s' for '[%s] %s'"),
+                             input, section, option);
+  else
+    return svn_error_createf(SVN_ERR_BAD_CONFIG_VALUE, NULL,
+                             _("Config error: invalid boolean "
+                               "value '%s' for '%s'"),
+                             input, option);
 
   return SVN_NO_ERROR;
 }
