@@ -2,17 +2,22 @@
  * mergeinfo-test.c -- test the mergeinfo functions
  *
  * ====================================================================
- * Copyright (c) 2006-2007 CollabNet.  All rights reserved.
+ *    Licensed to the Apache Software Foundation (ASF) under one
+ *    or more contributor license agreements.  See the NOTICE file
+ *    distributed with this work for additional information
+ *    regarding copyright ownership.  The ASF licenses this file
+ *    to you under the Apache License, Version 2.0 (the
+ *    "License"); you may not use this file except in compliance
+ *    with the License.  You may obtain a copy of the License at
  *
- * This software is licensed as described in the file COPYING, which
- * you should have received as part of this distribution.  The terms
- * are also available at http://subversion.tigris.org/license-1.html.
- * If newer versions of this license are posted there, you may use a
- * newer version instead, at your option.
+ *      http://www.apache.org/licenses/LICENSE-2.0
  *
- * This software consists of voluntary contributions made by many
- * individuals.  For exact contribution history, see the revision
- * history and logs, available at http://subversion.tigris.org/.
+ *    Unless required by applicable law or agreed to in writing,
+ *    software distributed under the License is distributed on an
+ *    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ *    KIND, either express or implied.  See the License for the
+ *    specific language governing permissions and limitations
+ *    under the License.
  * ====================================================================
  */
 
@@ -21,6 +26,8 @@
 #include <apr_hash.h>
 #include <apr_tables.h>
 
+#define SVN_DEPRECATED
+
 #include "svn_pools.h"
 #include "svn_types.h"
 #include "svn_mergeinfo.h"
@@ -110,7 +117,7 @@ verify_mergeinfo_parse(const char *input,
    -> merge ranges. */
 static apr_hash_t *info1, *info2;
 
-#define NBR_MERGEINFO_VALS 20
+#define NBR_MERGEINFO_VALS 24
 
 /* Valid mergeinfo values. */
 static const char * const mergeinfo_vals[NBR_MERGEINFO_VALS] =
@@ -139,7 +146,12 @@ static const char * const mergeinfo_vals[NBR_MERGEINFO_VALS] =
     "/gunther_branch:7-12*,1,5-10*",
     /* Adjacent rangelists of differing inheritability. */
     "/b5:5-53,1-4,54-90*",
-    "/c0:1-77,12-44"
+    "/c0:1-77,12-44",
+    /* Non-canonical paths. */
+    "/A/:7-8",
+    "/A///:7-8",
+    "/A/.:7-8",
+    "/A/./B:7-8"
   };
 /* Paths corresponding to mergeinfo_vals. */
 static const char * const mergeinfo_paths[NBR_MERGEINFO_VALS] =
@@ -154,7 +166,7 @@ static const char * const mergeinfo_paths[NBR_MERGEINFO_VALS] =
     "/patch-common::netasq-bpf.c",
     "/patch-common_netasq-bpf.c:",
     "/:patch:common:netasq:bpf.c",
-    
+
     "/trunk",
     "/trunk",
     "/trunk",
@@ -166,7 +178,13 @@ static const char * const mergeinfo_paths[NBR_MERGEINFO_VALS] =
     "/gunther_branch",
     "/gunther_branch",
     "/b5",
-    "/c0"
+    "/c0",
+
+    /* non-canonical paths converted to canonical */
+    "/A",
+    "/A",
+    "/A",
+    "/A/B"
   };
 /* First ranges from the paths identified by mergeinfo_paths. */
 static svn_merge_range_t mergeinfo_ranges[NBR_MERGEINFO_VALS][MAX_NBR_RANGES] =
@@ -193,21 +211,17 @@ static svn_merge_range_t mergeinfo_ranges[NBR_MERGEINFO_VALS][MAX_NBR_RANGES] =
     { {0, 1, TRUE}, {4, 12, FALSE} },
     { {0, 53, TRUE}, {53, 90, FALSE} },
     { {0, 77, TRUE} },
+    { {6, 8, TRUE} },
+    { {6, 8, TRUE} },
+    { {6, 8, TRUE} },
+    { {6, 8, TRUE} },
   };
 
 static svn_error_t *
-test_parse_single_line_mergeinfo(const char **msg,
-                                 svn_boolean_t msg_only,
-                                 svn_test_opts_t *opts,
-                                 apr_pool_t *pool)
+test_parse_single_line_mergeinfo(apr_pool_t *pool)
 {
   int i;
 
-  *msg = "parse single line mergeinfo";
-
-  if (msg_only)
-    return SVN_NO_ERROR;
-
   for (i = 0; i < NBR_MERGEINFO_VALS; i++)
     SVN_ERR(verify_mergeinfo_parse(mergeinfo_vals[i], mergeinfo_paths[i],
                                    mergeinfo_ranges[i], pool));
@@ -218,20 +232,12 @@ test_parse_single_line_mergeinfo(const char **msg,
 static const char *single_mergeinfo = "/trunk: 5,7-9,10,11,13,14";
 
 static svn_error_t *
-test_mergeinfo_dup(const char **msg,
-                   svn_boolean_t msg_only,
-                   svn_test_opts_t *opts,
-                   apr_pool_t *pool)
+test_mergeinfo_dup(apr_pool_t *pool)
 {
   apr_hash_t *orig_mergeinfo, *copied_mergeinfo;
   apr_pool_t *subpool;
   apr_array_header_t *rangelist;
 
-  *msg = "copy a mergeinfo data structure";
-
-  if (msg_only)
-    return SVN_NO_ERROR;
-
   /* Assure that copies which should be empty turn out that way. */
   subpool = svn_pool_create(pool);
   orig_mergeinfo = apr_hash_make(subpool);
@@ -257,19 +263,11 @@ test_mergeinfo_dup(const char **msg,
 }
 
 static svn_error_t *
-test_parse_combine_rangeinfo(const char **msg,
-                             svn_boolean_t msg_only,
-                             svn_test_opts_t *opts,
-                             apr_pool_t *pool)
+test_parse_combine_rangeinfo(apr_pool_t *pool)
 {
   apr_array_header_t *result;
   svn_merge_range_t *resultrange;
 
-  *msg = "parse single line mergeinfo and combine ranges";
-
-  if (msg_only)
-    return SVN_NO_ERROR;
-
   SVN_ERR(svn_mergeinfo_parse(&info1, single_mergeinfo, pool));
 
   if (apr_hash_count(info1) != 1)
@@ -343,17 +341,10 @@ static const char * const broken_mergeinfo_vals[NBR_BROKEN_MERGEINFO_VALS] =
   };
 
 static svn_error_t *
-test_parse_broken_mergeinfo(const char **msg,
-                            svn_boolean_t msg_only,
-                            svn_test_opts_t *opts,
-                            apr_pool_t *pool)
+test_parse_broken_mergeinfo(apr_pool_t *pool)
 {
   int i;
   svn_error_t *err;
-  *msg = "parse broken single line mergeinfo";
-
-  if (msg_only)
-    return SVN_NO_ERROR;
 
   /* Trigger some error(s) with mal-formed input. */
   for (i = 0; i < NBR_BROKEN_MERGEINFO_VALS; i++)
@@ -408,7 +399,7 @@ range_to_string(svn_merge_range_t *range,
    verified (e.g. "svn_rangelist_intersect"), while TYPE is a word
    describing what the ranges being examined represent. */
 static svn_error_t *
-verify_ranges_match(apr_array_header_t *actual_rangelist,
+verify_ranges_match(const apr_array_header_t *actual_rangelist,
                     svn_merge_range_t *expected_ranges, int nbr_expected,
                     const char *func_verified, const char *type,
                     apr_pool_t *pool)
@@ -462,10 +453,7 @@ verify_mergeinfo_deltas(apr_hash_t *deltas, svn_merge_range_t *expected_deltas,
 }
 
 static svn_error_t *
-test_diff_mergeinfo(const char **msg,
-                    svn_boolean_t msg_only,
-                    svn_test_opts_t *opts,
-                    apr_pool_t *pool)
+test_diff_mergeinfo(apr_pool_t *pool)
 {
   apr_hash_t *deleted, *added, *from, *to;
   svn_merge_range_t expected_rangelist_deletions[NBR_RANGELIST_DELTAS] =
@@ -473,10 +461,6 @@ test_diff_mergeinfo(const char **msg,
   svn_merge_range_t expected_rangelist_additions[NBR_RANGELIST_DELTAS] =
     { {1, 2, TRUE}, {4, 6, TRUE}, {12, 16, TRUE}, {29, 30, TRUE} };
 
-  *msg = "diff of mergeinfo";
-  if (msg_only)
-    return SVN_NO_ERROR;
-
   SVN_ERR(svn_mergeinfo_parse(&from, "/trunk: 1,3-4,7,9,11-12,31-34", pool));
   SVN_ERR(svn_mergeinfo_parse(&to, "/trunk: 1-6,12-16,30-32", pool));
   /* On /trunk: deleted (7, 9, 11, 33-34) and added (2, 5-6, 13-16, 30) */
@@ -493,19 +477,12 @@ test_diff_mergeinfo(const char **msg,
 }
 
 static svn_error_t *
-test_rangelist_reverse(const char **msg,
-                       svn_boolean_t msg_only,
-                       svn_test_opts_t *opts,
-                       apr_pool_t *pool)
+test_rangelist_reverse(apr_pool_t *pool)
 {
   apr_array_header_t *rangelist;
   svn_merge_range_t expected_rangelist[3] =
     { {10, 9, TRUE}, {7, 4, TRUE}, {3, 2, TRUE} };
 
-  *msg = "reversal of rangelist";
-  if (msg_only)
-    return SVN_NO_ERROR;
-
   SVN_ERR(svn_mergeinfo_parse(&info1, "/trunk: 3,5-7,10", pool));
   rangelist = apr_hash_get(info1, "/trunk", APR_HASH_KEY_STRING);
 
@@ -516,10 +493,7 @@ test_rangelist_reverse(const char **msg,
 }
 
 static svn_error_t *
-test_rangelist_intersect(const char **msg,
-                         svn_boolean_t msg_only,
-                         svn_test_opts_t *opts,
-                         apr_pool_t *pool)
+test_rangelist_intersect(apr_pool_t *pool)
 {
   apr_array_header_t *rangelist1, *rangelist2, *intersection;
 
@@ -532,10 +506,6 @@ test_rangelist_intersect(const char **msg,
     { {0, 1, TRUE}, {2, 4, TRUE}, {11, 12, TRUE}, {30, 32, FALSE},
       {39, 42, TRUE} };
 
-  *msg = "intersection of rangelists";
-  if (msg_only)
-    return SVN_NO_ERROR;
-
   SVN_ERR(svn_mergeinfo_parse(&info1, "/trunk: 1-6,12-16,30-32*,40-42", pool));
   SVN_ERR(svn_mergeinfo_parse(&info2, "/trunk: 1,3-4*,7,9,11-12,31-34*,38-44",
                               pool));
@@ -591,20 +561,13 @@ test_rangelist_intersect(const char **msg,
 }
 
 static svn_error_t *
-test_mergeinfo_intersect(const char **msg,
-                         svn_boolean_t msg_only,
-                         svn_test_opts_t *opts,
-                         apr_pool_t *pool)
+test_mergeinfo_intersect(apr_pool_t *pool)
 {
   svn_merge_range_t expected_intersection[3] =
     { {0, 1, TRUE}, {2, 4, TRUE}, {11, 12, TRUE} };
   apr_array_header_t *rangelist;
   apr_hash_t *intersection;
 
-  *msg = "intersection of mergeinfo";
-  if (msg_only)
-    return SVN_NO_ERROR;
-
   SVN_ERR(svn_mergeinfo_parse(&info1, "/trunk: 1-6,12-16\n/foo: 31", pool));
   SVN_ERR(svn_mergeinfo_parse(&info2, "/trunk: 1,3-4,7,9,11-12", pool));
 
@@ -620,10 +583,7 @@ test_mergeinfo_intersect(const char **msg,
 }
 
 static svn_error_t *
-test_merge_mergeinfo(const char **msg,
-                     svn_boolean_t msg_only,
-                     svn_test_opts_t *opts,
-                     apr_pool_t *pool)
+test_merge_mergeinfo(apr_pool_t *pool)
 {
   int i;
 
@@ -738,10 +698,6 @@ test_merge_mergeinfo(const char **msg,
           {"/fred",    { {0, 18, TRUE} } } } }
     };
 
-  *msg = "merging of mergeinfo hashs";
-  if (msg_only)
-    return SVN_NO_ERROR;
-
   for (i = 0; i < NBR_MERGEINFO_MERGES; i++)
     {
       int j;
@@ -811,10 +767,7 @@ test_merge_mergeinfo(const char **msg,
 }
 
 static svn_error_t *
-test_remove_rangelist(const char **msg,
-                      svn_boolean_t msg_only,
-                      svn_test_opts_t *opts,
-                      apr_pool_t *pool)
+test_remove_rangelist(apr_pool_t *pool)
 {
   int i, j;
   svn_error_t *err, *child_err;
@@ -882,7 +835,6 @@ test_remove_rangelist(const char **msg,
                               2, { {4, 8, TRUE }, {9, 100, TRUE }}}
     };
 
-  *msg = "remove rangelists";
   err = child_err = SVN_NO_ERROR;
   for (j = 0; j < 2; j++)
     {
@@ -992,7 +944,8 @@ test_remove_rangelist(const char **msg,
 /* Random number seed. */
 static apr_uint32_t random_rev_array_seed;
 
-/* Fill 3/4 of the array with 1s. */
+/* Set a random 3/4-ish of the elements of array REVS[RANDOM_REV_ARRAY_LENGTH]
+ * to TRUE and the rest to FALSE. */
 static void
 randomly_fill_rev_array(svn_boolean_t *revs)
 {
@@ -1004,6 +957,8 @@ randomly_fill_rev_array(svn_boolean_t *revs)
     }
 }
 
+/* Set *RANGELIST to a rangelist representing the revisions that are marked
+ * with TRUE in the array REVS[RANDOM_REV_ARRAY_LENGTH]. */
 static svn_error_t *
 rev_array_to_rangelist(apr_array_header_t **rangelist,
                        svn_boolean_t *revs,
@@ -1033,18 +988,11 @@ rev_array_to_rangelist(apr_array_header_t **rangelist,
 }
 
 static svn_error_t *
-test_rangelist_remove_randomly(const char **msg,
-                               svn_boolean_t msg_only,
-                               svn_test_opts_t *opts,
-                               apr_pool_t *pool)
+test_rangelist_remove_randomly(apr_pool_t *pool)
 {
   int i;
   apr_pool_t *iterpool;
 
-  *msg = "test rangelist remove with random data";
-  if (msg_only)
-    return SVN_NO_ERROR;
-
   random_rev_array_seed = (apr_uint32_t) apr_time_now();
 
   iterpool = svn_pool_create(pool);
@@ -1065,6 +1013,9 @@ test_rangelist_remove_randomly(const char **msg,
 
       randomly_fill_rev_array(first_revs);
       randomly_fill_rev_array(second_revs);
+      /* There is no change numbered "r0" */
+      first_revs[0] = FALSE;
+      second_revs[0] = FALSE;
       for (j = 0; j < RANDOM_REV_ARRAY_LENGTH; j++)
         expected_revs[j] = second_revs[j] && !first_revs[j];
 
@@ -1095,18 +1046,11 @@ test_rangelist_remove_randomly(const char **msg,
 }
 
 static svn_error_t *
-test_rangelist_intersect_randomly(const char **msg,
-                                  svn_boolean_t msg_only,
-                                  svn_test_opts_t *opts,
-                                  apr_pool_t *pool)
+test_rangelist_intersect_randomly(apr_pool_t *pool)
 {
   int i;
   apr_pool_t *iterpool;
 
-  *msg = "test rangelist intersect with random data";
-  if (msg_only)
-    return SVN_NO_ERROR;
-
   random_rev_array_seed = (apr_uint32_t) apr_time_now();
 
   iterpool = svn_pool_create(pool);
@@ -1127,6 +1071,9 @@ test_rangelist_intersect_randomly(const char **msg,
 
       randomly_fill_rev_array(first_revs);
       randomly_fill_rev_array(second_revs);
+      /* There is no change numbered "r0" */
+      first_revs[0] = FALSE;
+      second_revs[0] = FALSE;
       for (j = 0; j < RANDOM_REV_ARRAY_LENGTH; j++)
         expected_revs[j] = second_revs[j] && first_revs[j];
 
@@ -1158,19 +1105,12 @@ test_rangelist_intersect_randomly(const char **msg,
 
 /* ### Share code with test_diff_mergeinfo() and test_remove_rangelist(). */
 static svn_error_t *
-test_remove_mergeinfo(const char **msg,
-                      svn_boolean_t msg_only,
-                      svn_test_opts_t *opts,
-                      apr_pool_t *pool)
+test_remove_mergeinfo(apr_pool_t *pool)
 {
   apr_hash_t *output, *whiteboard, *eraser;
   svn_merge_range_t expected_rangelist_remainder[NBR_RANGELIST_DELTAS] =
     { {6, 7, TRUE}, {8, 9, TRUE}, {10, 11, TRUE}, {32, 34, TRUE} };
 
-  *msg = "remove of mergeinfo";
-  if (msg_only)
-    return SVN_NO_ERROR;
-
   SVN_ERR(svn_mergeinfo_parse(&whiteboard,
                               "/trunk: 1,3-4,7,9,11-12,31-34", pool));
   SVN_ERR(svn_mergeinfo_parse(&eraser, "/trunk: 1-6,12-16,30-32", pool));
@@ -1185,20 +1125,12 @@ test_remove_mergeinfo(const char **msg,
 #undef NBR_RANGELIST_DELTAS
 
 static svn_error_t *
-test_rangelist_to_string(const char **msg,
-                         svn_boolean_t msg_only,
-                         svn_test_opts_t *opts,
-                         apr_pool_t *pool)
+test_rangelist_to_string(apr_pool_t *pool)
 {
   apr_array_header_t *result;
   svn_string_t *output;
   svn_string_t *expected = svn_string_create("3,5,7-11,13-14", pool);
 
-  *msg = "turning rangelist back into a string";
-
-  if (msg_only)
-    return SVN_NO_ERROR;
-
   SVN_ERR(svn_mergeinfo_parse(&info1, mergeinfo1, pool));
 
   result = apr_hash_get(info1, "/trunk", APR_HASH_KEY_STRING);
@@ -1214,20 +1146,12 @@ test_rangelist_to_string(const char **msg,
 }
 
 static svn_error_t *
-test_mergeinfo_to_string(const char **msg,
-                         svn_boolean_t msg_only,
-                         svn_test_opts_t *opts,
-                         apr_pool_t *pool)
+test_mergeinfo_to_string(apr_pool_t *pool)
 {
   svn_string_t *output;
   svn_string_t *expected;
   expected = svn_string_create("/fred:8-10\n/trunk:3,5,7-11,13-14", pool);
 
-  *msg = "turning mergeinfo back into a string";
-
-  if (msg_only)
-    return SVN_NO_ERROR;
-
   SVN_ERR(svn_mergeinfo_parse(&info1, mergeinfo1, pool));
 
   SVN_ERR(svn_mergeinfo_to_string(&output, info1, pool));
@@ -1247,7 +1171,7 @@ test_mergeinfo_to_string(const char **msg,
                APR_HASH_KEY_STRING,
                apr_hash_get(info1, "/trunk", APR_HASH_KEY_STRING));
   SVN_ERR(svn_mergeinfo_to_string(&output, info2, pool));
-  
+
   if (svn_string_compare(expected, output) != TRUE)
     return fail(pool, "Mergeinfo string not what we expected");
 
@@ -1256,10 +1180,7 @@ test_mergeinfo_to_string(const char **msg,
 
 
 static svn_error_t *
-test_rangelist_merge(const char **msg,
-                     svn_boolean_t msg_only,
-                     svn_test_opts_t *opts,
-                     apr_pool_t *pool)
+test_rangelist_merge(apr_pool_t *pool)
 {
   int i;
   svn_error_t *err, *child_err;
@@ -1275,7 +1196,7 @@ test_rangelist_merge(const char **msg,
     svn_merge_range_t expected_merge[6];
   };
 
-  #define SIZE_OF_RANGE_MERGE_TEST_ARRAY 59
+  #define SIZE_OF_RANGE_MERGE_TEST_ARRAY 68
   /* The actual test data. */
   struct rangelist_merge_test_data test_data[SIZE_OF_RANGE_MERGE_TEST_ARRAY] =
     {
@@ -1360,6 +1281,29 @@ test_rangelist_merge(const char **msg,
       {"/A: 2-17", "/A: 1-5*,7*,12-13*", 2,
        {{0, 1, FALSE}, {1, 17, TRUE}}},
 
+      {"/A: 3-4*,10-15,20", "/A: 5-60*", 5,
+       {{2, 9, FALSE}, {9, 15, TRUE}, {15, 19, FALSE},{19, 20, TRUE},
+        {20, 60, FALSE}}},
+
+      {"/A: 5-60*", "/A: 3-4*,10-15,20", 5,
+       {{2, 9, FALSE}, {9, 15, TRUE}, {15, 19, FALSE},{19, 20, TRUE},
+        {20, 60, FALSE}}},
+
+      {"/A: 3-4*,50-100*", "/A: 5-60*", 1, {{2, 100, FALSE}}},
+
+      {"/A: 5-60*", "/A: 3-4*,50-100*", 1, {{2, 100, FALSE}}},
+
+      {"/A: 3-4*,50-100", "/A: 5-60*", 2, {{2, 49, FALSE}, {49, 100, TRUE}}},
+
+      {"/A: 5-60*", "/A: 3-4*,50-100", 2, {{2, 49, FALSE}, {49, 100, TRUE}}},
+
+      {"/A: 3-4,50-100*", "/A: 5-60", 2, {{2, 60, TRUE}, {60, 100, FALSE}}},
+
+      {"/A: 5-60", "/A: 3-4,50-100*", 2, {{2, 60, TRUE}, {60, 100, FALSE}}},
+
+      {"/A: 5,9,11-15,17,200-300,999", "/A: 7-50", 4,
+       {{4, 5, TRUE}, {6, 50, TRUE}, {199, 300, TRUE}, {998, 999, TRUE}}},
+
       /* A rangelist merged with an empty rangelist should equal the
          non-empty rangelist but in compacted form. */
       {"/A: 1-44,45,46,47-50",       "",  1, {{ 0, 50, TRUE }}},
@@ -1375,9 +1319,6 @@ test_rangelist_merge(const char **msg,
          please, an empty rangelist. */
       {"", "", 0, {{0, 0, FALSE}}}
     };
-  *msg = "merge of rangelists";
-  if (msg_only)
-    return SVN_NO_ERROR;
 
   err = child_err = SVN_NO_ERROR;
   for (i = 0; i < SIZE_OF_RANGE_MERGE_TEST_ARRAY; i++)
@@ -1438,10 +1379,7 @@ test_rangelist_merge(const char **msg,
 }
 
 static svn_error_t *
-test_rangelist_diff(const char **msg,
-                    svn_boolean_t msg_only,
-                    svn_test_opts_t *opts,
-                    apr_pool_t *pool)
+test_rangelist_diff(apr_pool_t *pool)
 {
   int i;
   svn_error_t *err, *child_err;
@@ -1596,10 +1534,6 @@ test_rangelist_diff(const char **msg,
        0, { { 0, 0, FALSE } } },
     };
 
-  *msg = "diff of rangelists";
-  if (msg_only)
-    return SVN_NO_ERROR;
-
   err = child_err = SVN_NO_ERROR;
   for (i = 0; i < SIZE_OF_RANGE_DIFF_TEST_ARRAY; i++)
     {
@@ -1665,28 +1599,140 @@ test_rangelist_diff(const char **msg,
     }
   return err;
 }
+
+
+/* Test data structure for test_remove_prefix_from_catalog(). */
+struct catalog_bits
+{
+  const char *orig_path;
+  const char *new_path;
+  const char *mergeinfo;
+};
+
+
+/* Helper for test_remove_prefix_from_catalog(). */
+static svn_error_t *
+remove_prefix_helper(struct catalog_bits *test_data,
+                     const char *prefix_path,
+                     apr_pool_t *pool)
+{
+  svn_mergeinfo_catalog_t in_catalog, out_catalog, exp_out_catalog;
+  apr_hash_index_t *hi;
+  int i = 0;
+
+  in_catalog = apr_hash_make(pool);
+  exp_out_catalog = apr_hash_make(pool);
+  while (test_data[i].orig_path)
+    {
+      struct catalog_bits data = test_data[i];
+      const char *orig_path = apr_pstrdup(pool, data.orig_path);
+      const char *new_path = apr_pstrdup(pool, data.new_path);
+      svn_mergeinfo_t mergeinfo;
+      SVN_ERR(svn_mergeinfo_parse(&mergeinfo, data.mergeinfo, pool));
+      apr_hash_set(in_catalog, orig_path, APR_HASH_KEY_STRING, mergeinfo);
+      apr_hash_set(exp_out_catalog, new_path, APR_HASH_KEY_STRING, mergeinfo);
+      i++;
+    }
+  SVN_ERR(svn_mergeinfo__remove_prefix_from_catalog(&out_catalog, in_catalog,
+                                                    prefix_path, pool));
+  if (apr_hash_count(exp_out_catalog) != apr_hash_count(out_catalog))
+    return svn_error_create(SVN_ERR_TEST_FAILED, 0,
+                            "Got unexpected number of catalog entries");
+  for (hi = apr_hash_first(pool, out_catalog); hi; hi = apr_hash_next(hi))
+    {
+      const void *path;
+      apr_ssize_t path_len;
+      void *out_mergeinfo, *exp_out_mergeinfo;
+      apr_hash_this(hi, &path, &path_len, &out_mergeinfo);
+      exp_out_mergeinfo = apr_hash_get(exp_out_catalog, path, path_len);
+      if (! exp_out_mergeinfo)
+        return svn_error_createf(SVN_ERR_TEST_FAILED, 0,
+                                 "Found unexpected key '%s' in catalog",
+                                 (const char *)path);
+      if (exp_out_mergeinfo != out_mergeinfo)
+        return svn_error_create(SVN_ERR_TEST_FAILED, 0,
+                                "Detected value tampering in catalog");
+    }
+  return SVN_NO_ERROR;
+}
+
+static svn_error_t *
+test_remove_prefix_from_catalog(apr_pool_t *pool)
+{
+  apr_pool_t *subpool = svn_pool_create(pool);
+
+  /* For testing the remove of the prefix "/trunk"  */
+  struct catalog_bits test_data_1[] =
+    {
+      { "/trunk",           "",          "/A:1" },
+      { "/trunk/foo",       "foo",       "/A/foo:1,3*" },
+      { "/trunk/foo/bar",   "foo/bar",   "/A/foo:1-4" },
+      { "/trunk/baz",       "baz",       "/A/baz:2" },
+      { NULL, NULL, NULL }
+    };
+
+  /* For testing the remove of the prefix "/"  */
+  struct catalog_bits test_data_2[] =
+    {
+      { "/",                "",                "/:2" },
+      { "/trunk",           "trunk",           "/A:1" },
+      { "/trunk/foo",       "trunk/foo",       "/A/foo:1,3*" },
+      { "/trunk/foo/bar",   "trunk/foo/bar",   "/A/foo:1-4" },
+      { "/trunk/baz",       "trunk/baz",       "/A/baz:2" },
+      { NULL, NULL, NULL }
+    };
+
+  svn_pool_clear(subpool);
+  SVN_ERR(remove_prefix_helper(test_data_1, "/trunk", subpool));
+
+  svn_pool_clear(subpool);
+  SVN_ERR(remove_prefix_helper(test_data_2, "/", subpool));
+
+  svn_pool_destroy(subpool);
+  return SVN_NO_ERROR;
+}
+
 
 /* The test table.  */
 
 struct svn_test_descriptor_t test_funcs[] =
   {
     SVN_TEST_NULL,
-    SVN_TEST_PASS(test_parse_single_line_mergeinfo),
-    SVN_TEST_PASS(test_mergeinfo_dup),
-    SVN_TEST_PASS(test_parse_combine_rangeinfo),
-    SVN_TEST_PASS(test_parse_broken_mergeinfo),
-    SVN_TEST_PASS(test_remove_rangelist),
-    SVN_TEST_PASS(test_rangelist_remove_randomly),
-    SVN_TEST_PASS(test_remove_mergeinfo),
-    SVN_TEST_PASS(test_rangelist_reverse),
-    SVN_TEST_PASS(test_rangelist_intersect),
-    SVN_TEST_PASS(test_rangelist_intersect_randomly),
-    SVN_TEST_PASS(test_diff_mergeinfo),
-    SVN_TEST_PASS(test_merge_mergeinfo),
-    SVN_TEST_PASS(test_mergeinfo_intersect),
-    SVN_TEST_PASS(test_rangelist_to_string),
-    SVN_TEST_PASS(test_mergeinfo_to_string),
-    SVN_TEST_PASS(test_rangelist_merge),
-    SVN_TEST_PASS(test_rangelist_diff),
+    SVN_TEST_PASS2(test_parse_single_line_mergeinfo,
+                   "parse single line mergeinfo"),
+    SVN_TEST_PASS2(test_mergeinfo_dup,
+                   "copy a mergeinfo data structure"),
+    SVN_TEST_PASS2(test_parse_combine_rangeinfo,
+                   "parse single line mergeinfo and combine ranges"),
+    SVN_TEST_PASS2(test_parse_broken_mergeinfo,
+                   "parse broken single line mergeinfo"),
+    SVN_TEST_PASS2(test_remove_rangelist,
+                   "remove rangelists"),
+    SVN_TEST_PASS2(test_rangelist_remove_randomly,
+                   "test rangelist remove with random data"),
+    SVN_TEST_PASS2(test_remove_mergeinfo,
+                   "remove of mergeinfo"),
+    SVN_TEST_PASS2(test_rangelist_reverse,
+                   "reversal of rangelist"),
+    SVN_TEST_PASS2(test_rangelist_intersect,
+                   "intersection of rangelists"),
+    SVN_TEST_PASS2(test_rangelist_intersect_randomly,
+                   "test rangelist intersect with random data"),
+    SVN_TEST_PASS2(test_diff_mergeinfo,
+                   "diff of mergeinfo"),
+    SVN_TEST_PASS2(test_merge_mergeinfo,
+                   "merging of mergeinfo hashs"),
+    SVN_TEST_PASS2(test_mergeinfo_intersect,
+                   "intersection of mergeinfo"),
+    SVN_TEST_PASS2(test_rangelist_to_string,
+                   "turning rangelist back into a string"),
+    SVN_TEST_PASS2(test_mergeinfo_to_string,
+                   "turning mergeinfo back into a string"),
+    SVN_TEST_PASS2(test_rangelist_merge,
+                   "merge of rangelists"),
+    SVN_TEST_PASS2(test_rangelist_diff,
+                   "diff of rangelists"),
+    SVN_TEST_PASS2(test_remove_prefix_from_catalog,
+                   "removal of prefix paths from catalog keys"),
     SVN_TEST_NULL
   };
